# -*- coding: utf-8 -*-
"""
KM3NeT Data Definitions v3.0.0
https://git.km3net.de/common/km3net-dataformat
"""

# w2list_km3buu
data = dict(
    W2LIST_KM3BUU_PS=0,
    W2LIST_KM3BUU_EG=1,
    W2LIST_KM3BUU_XSEC_MEAN=2,
    W2LIST_KM3BUU_COLUMN_DEPTH=3,
    W2LIST_KM3BUU_P_EARTH=4,
    W2LIST_KM3BUU_WATER_INT_LEN=5,
    W2LIST_KM3BUU_BX=7,
    W2LIST_KM3BUU_BY=8,
    W2LIST_KM3BUU_ICHAN=9,
    W2LIST_KM3BUU_CC=10,
    W2LIST_KM3BUU_XSEC=13,
    W2LIST_KM3BUU_DXSEC=14,
    W2LIST_KM3BUU_TARGETA=15,
    W2LIST_KM3BUU_TARGETZ=16,
    W2LIST_KM3BUU_VERINCAN=17,
    W2LIST_KM3BUU_LEPINCAN=18,
    W2LIST_KM3BUU_GIBUU_WEIGHT=23,
    W2LIST_KM3BUU_GIBUU_SCAT_TYPE=24,
)



# -*- coding: utf-8 -*-
"""
KM3NeT Data Definitions v3.0.0
https://git.km3net.de/common/km3net-dataformat
"""

# applications
data = dict(
    APPLICATION_GENHEN=   "GENHEN"   ,
    APPLICATION_GSEAGEN=  "gSeaGen"  ,
    APPLICATION_MUPAGE=   "MUPAGE"   ,
    APPLICATION_CORSIKA=  "CORSIKA"  ,
    APPLICATION_KM3BUU=   "KM3BUU"   ,
    APPLICATION_KM3=      "km3"      ,
    APPLICATION_KM3SIM=   "KM3Sim"   ,
    APPLICATION_JSIRENE=  "JSirene"  ,
)



# -*- coding: utf-8 -*-
"""
KM3NeT Data Definitions v3.0.0
https://git.km3net.de/common/km3net-dataformat
"""

# root
data = dict(
    TTREE_ONLINE_TIMESLICE=    "KM3NET_TIMESLICE"   ,
    TTREE_ONLINE_TIMESLICEL0=  "KM3NET_TIMESLICE_L0",
    TTREE_ONLINE_TIMESLICEL1=  "KM3NET_TIMESLICE_L1",
    TTREE_ONLINE_TIMESLICEL2=  "KM3NET_TIMESLICE_L2",
    TTREE_ONLINE_TIMESLICESN=  "KM3NET_TIMESLICE_SN",
    TTREE_ONLINE_SUMMARYSLICE= "KM3NET_SUMMARYSLICE",
    TTREE_ONLINE_EVENT=        "KM3NET_EVENT"       ,
    TTREE_OFFLINE_EVENT=       "E"                  ,
    TBRANCH_ONLINE_TIMESLICE=    "KM3NET_TIMESLICE"   ,
    TBRANCH_ONLINE_TIMESLICEL0=  "km3net_timeslice_L0",
    TBRANCH_ONLINE_TIMESLICEL1=  "km3net_timeslice_L1",
    TBRANCH_ONLINE_TIMESLICEL2=  "km3net_timeslice_L2",
    TBRANCH_ONLINE_TIMESLICESN=  "km3net_timeslice_SN",
    TBRANCH_ONLINE_SUMMARYSLICE= "KM3NET_SUMMARYSLICE",
    TBRANCH_ONLINE_EVENT=        "KM3NET_EVENT"       ,
    TBRANCH_OFFLINE_EVENT=       "Evt"                ,
    COMPRESSION_LEVEL_ONLINE_TIMESLICE=    0,
    COMPRESSION_LEVEL_ONLINE_TIMESLICEL0=  0,
    COMPRESSION_LEVEL_ONLINE_TIMESLICEL1=  2,
    COMPRESSION_LEVEL_ONLINE_TIMESLICEL2=  2,
    COMPRESSION_LEVEL_ONLINE_TIMESLICESN=  2,
    COMPRESSION_LEVEL_ONLINE_SUMMARYSLICE= 1,
    COMPRESSION_LEVEL_ONLINE_EVENT=        0,
    COMPRESSION_LEVEL_OFFLINE_EVENT=       1,
    BASKET_SIZE_ONLINE_TIMESLICE=      5000000,
    BASKET_SIZE_ONLINE_TIMESLICEL0=  500000000,
    BASKET_SIZE_ONLINE_TIMESLICEL1=    5000000,
    BASKET_SIZE_ONLINE_TIMESLICEL2=    5000000,
    BASKET_SIZE_ONLINE_TIMESLICESN=    5000000,
    BASKET_SIZE_ONLINE_SUMMARYSLICE=   5000000,
    BASKET_SIZE_ONLINE_EVENT=          5000000,
    BASKET_SIZE_OFFLINE_EVENT=         5000000,
    SPLIT_LEVEL_ONLINE_TIMESLICE=    1,
    SPLIT_LEVEL_ONLINE_TIMESLICEL0=  2,
    SPLIT_LEVEL_ONLINE_TIMESLICEL1=  2,
    SPLIT_LEVEL_ONLINE_TIMESLICEL2=  2,
    SPLIT_LEVEL_ONLINE_TIMESLICESN=  2,
    SPLIT_LEVEL_ONLINE_SUMMARYSLICE= 1,
    SPLIT_LEVEL_ONLINE_EVENT=        1,
    SPLIT_LEVEL_OFFLINE_EVENT=       4,
)



# -*- coding: utf-8 -*-
"""
KM3NeT Data Definitions v3.0.0
https://git.km3net.de/common/km3net-dataformat
"""

# trkmembers
data = dict(
    TRK_MOTHER_UNDEFINED=-1,
    TRK_MOTHER_NONE=-2,
    TRK_ST_UNDEFINED=0,
    TRK_ST_FINALSTATE=1,
    TRK_ST_PRIMARYNEUTRINO=100,
    TRK_ST_PRIMARYCOSMIC=200,
    TRK_ST_MUONBUNDLE=300,
    TRK_ST_ININUCLEI=5,
    TRK_ST_INTERSTATE=2,
    TRK_ST_DECSTATE=3,
    TRK_ST_NUCTGT=11,
    TRK_ST_PREHAD=12,
    TRK_ST_PRERES=13,
    TRK_ST_HADNUC=14,
    TRK_ST_NUCLREM=15,
    TRK_ST_NUCLCLT=16,
    TRK_ST_FAKECORSIKA=21,
    TRK_ST_FAKECORSIKA_DEC_MU_START=22,
    TRK_ST_FAKECORSIKA_DEC_MU_END=23,
    TRK_ST_FAKECORSIKA_ETA_2GAMMA=24,
    TRK_ST_FAKECORSIKA_ETA_3PI0=25,
    TRK_ST_FAKECORSIKA_ETA_PIP_PIM_PI0=26,
    TRK_ST_FAKECORSIKA_ETA_2PI_GAMMA=27,
    TRK_ST_FAKECORSIKA_CHERENKOV_GAMMA=28,
    TRK_ST_PROPLEPTON=1001,
    TRK_ST_PROPDECLEPTON=2001,
    PDG_MUONBUNDLE=81,
)



# -*- coding: utf-8 -*-
"""
KM3NeT Data Definitions v3.0.0
https://git.km3net.de/common/km3net-dataformat
"""

# daqdatatypes
data = dict(
    DAQSUPERFRAME=101,
    DAQSUMMARYFRAME=201,
    DAQTIMESLICE=1001,
    DAQTIMESLICEL0=1002,
    DAQTIMESLICEL1=1003,
    DAQTIMESLICEL2=1004,
    DAQTIMESLICESN=1005,
    DAQSUMMARYSLICE=2001,
    DAQEVENT=10001,
)



"""
This tookit provides an interface to read the raw acoustic binary data tipe as
produced by the Acoustic Data Filter (ADF).

The acoustic signals are acquired by the CLB with a 195312.5 Hz sampling
frequency (F_S) corresponding to a period of 5.12 microseconds (16 ns * 320).

The acoustic data filter processes the stream in segments (windows) of given
size. Two consecutive segments overlap by a given number of samples. These two
parameters are specified in the ADF configuration as:

  * DAQ_ADF_ANALYSIS_WINDOW_SIZE (typ.: 131072)
  * DAQ_ADF_ANALYSIS_WINDOW_OVERLAP (typ.: 7812)

Typical values are indicated but may change in the future.

When the ADF is set to dump the raw data, the overlapping segment is omitted
so the length of the data chunk is

  FRAME_LENGTH = DAQ_ADF_ANALYSIS_WINDOW_SIZE - DAQ_ADF_ANALYSIS_WINDOW_OVERLAP

Summary description of the raw acoustic data format.

- 4B: UTC seconds (UNIX timestamp);
- 4B: number of 16ns cycles;
- 4B: referred to as 'samples' corresponds to DAQ_ADF_ANALYSIS_WINDOW_SIZE
      (typ.: 131072).

Follows a sequence of 4B * FRAME_LENGTH audio samples. Each sample is 32 bit
float PCM value. FRAME_LENGTH cannot be reconstructed from
DAQ_ADF_ANALYSIS_WINDOW_SIZE without knowing DAQ_ADF_ANALYSIS_WINDOW_OVERLAP
so it has to be set by the user.

FRAME_LENGTH should be constant within the same file, so this approach will hold
(FRAME_LENGTH is configurable in the constructor).

Note: each file contains data from a single transducer and the DOM or base ID
is stored in the filename only. This is not a very good design but here it is
tentatively dealt with.

WARNING: data should be in general expected as ordered but may not be contiguous
in time.
"""
import numpy as np


F_S = 195312.5  # sampling frequency of the acoustic stream in the CLB


def get_dtype(FRAME_LENGTH):
    """Returns the data layout corresponding to FRAME_LENGTH"""
    DATA_TYPE = np.dtype(
        [
            ("utc_seconds", np.uint32),
            ("16ns_cycles", np.uint32),
            ("samples", np.uint32),
            ("frame", np.float32, FRAME_LENGTH),
        ]
    )
    return DATA_TYPE


class RawAcousticReader:
    def __init__(self, filepath, FRAME_LENGTH=123260):

        self.FRAME_LENGTH = FRAME_LENGTH
        DATA_TYPE = get_dtype(FRAME_LENGTH)

        with open(filepath) as acoufile:
            self._data = np.fromfile(acoufile, dtype=DATA_TYPE)
            """ extract CLB id from filename """
            """ split the extension and scan backwards the path """
            self._id = filepath.split(".")[-2][-24 : -16 + 1]

    @property
    def id(self):
        return self._id

    @property
    def pcm(self):
        """Get PCM data concatenating all frames. Data may not be not contiguous."""
        return self._data["frame"].flatten()

    @property
    def timestamps(self):
        return self._data["utc_seconds"], self._data["16ns_cycles"]

    @property
    def timebase(self):
        """
        Constructs sequence of times corresponding to each sample.
        For convenience, the time is stored as a double precision float.
        The resolution is not fixed (which is sub-optimal) but should always
        allow exact representation of the sample time.
        """
        sample_interval = 1 / F_S
        frame_duration = self.FRAME_LENGTH * sample_interval
        time_axis = np.arange(0, frame_duration, sample_interval)
        start_frame = self._data[0]
        n_samples = self.FRAME_LENGTH * len(self._data)

        timebase = np.zeros(n_samples, dtype=np.float64)

        for i, frame in enumerate(self._data):
            sample_range = slice(i * self.FRAME_LENGTH, (i + 1) * self.FRAME_LENGTH)
            timebase[sample_range] = (
                frame["utc_seconds"] + 16e-9 * frame["16ns_cycles"] + time_axis
            )

        return timebase

    def to_wav(self, filepath, gain_dB=0.0):
        """
        Write as wave, with optional gain.
        """
        from scipy.io import wavfile

        pcm = self.pcm
        if gain_dB != 0.0:
            pcm *= 10 ** (0.1 * gain_dB)
        wavfile.write(filepath, int(F_S), pcm)



# -*- coding: utf-8 -*-
"""
KM3NeT Data Definitions v3.0.0
https://git.km3net.de/common/km3net-dataformat
"""

# w2list_genhen
data = dict(
    W2LIST_GENHEN_GLOBAL_GEN_WEIGHT=0,
    W2LIST_GENHEN_EG=1,
    W2LIST_GENHEN_SIG=2,
    W2LIST_GENHEN_COLUMN_DEPTH=3,
    W2LIST_GENHEN_P_EARTH=4,
    W2LIST_GENHEN_REFF=5,
    W2LIST_GENHEN_BX=7,
    W2LIST_GENHEN_BY=8,
    W2LIST_GENHEN_ICHAN=9,
    W2LIST_GENHEN_CC=10,
)
